@@ -101,11 +101,46 @@ public:
 
 static TimeFormats timeformats;
 
-double stringToTime(const std::string s) {
+// given a ptime object, return (fractional) seconds since epoch
+// account for localtime, and also account for dst
+double ptToDouble(const bt::ptime & pt) {
 
     //This local adjustor depends on the machine TZ settings-- highly dangerous!
     typedef boost::date_time::c_local_adjustor<bt::ptime> local_adj;
 
+    const bt::ptime timet_start(boost::gregorian::date(1970,1,1));
+    const bt::ptime local_timet_start = local_adj::utc_to_local(timet_start);
+
+    // seconds sincr epoch (in local time) -- misses DST adjustment
+    bt::time_duration tdiff = pt - local_timet_start;
+    //Rcpp::Rcout << "tdiff is " << tdiff << std::endl;
+    //Rcpp::Rcout << "pt is " << pt << std::endl;
+
+    // hack-ish: go back to struct tm to use its tm_isdst field
+    time_t secsSinceEpoch = tdiff.total_seconds();
+    struct tm* localAsTm = localtime(&secsSinceEpoch);
+    //Rcpp::Rcout << "Adj is " << localAsTm->tm_isdst << std::endl;
+
+    // Define BOOST_DATE_TIME_POSIX_TIME_STD_CONFIG to use nanoseconds
+    // (and then use diff.total_nanoseconds()/1.0e9;  instead)
+    //
+    // note dst correction here -- needed as UTC offset is correct but does not
+    // contain the additional DST adjustment
+    double totsec = tdiff.total_microseconds()/1.0e6, dstadj = 0;
+#if defined(_WIN32)
+    if (totsec > 0) {           // on Windows, for dates before 1970-01-01: segfault
+        dstadj = localAsTm->tm_isdst*60*60;
+    }
+#else
+    dstadj = localAsTm->tm_isdst*60*60;
+#endif
+    return totsec - dstadj;
+}
+
+// given a string with a (date)time object, try all formats til we parse one
+// conversion of ptime object to double done by ptToDouble()
+double stringToTime(const std::string s) {
+
     bt::ptime pt, ptbase;
 
     // loop over formats and try them til one fits
@@ -116,29 +151,7 @@ double stringToTime(const std::string s) {
         is >> pt;
     }
 
-    if (pt == ptbase) {
-        return NAN;
-    } else {
-        const bt::ptime timet_start(boost::gregorian::date(1970,1,1));
-        const bt::ptime local_timet_start = local_adj::utc_to_local(timet_start);
-
-        // seconds sincr epoch (in local time) -- misses DST adjustment
-        bt::time_duration tdiff = pt - local_timet_start;
-        //Rcpp::Rcout << "tdiff is " << tdiff << std::endl;
-        //Rcpp::Rcout << "pt is " << pt << std::endl;
-
-        // hack-ish: go back to struct tm to use its tm_isdst field
-        time_t secsSinceEpoch = tdiff.total_seconds();
-        struct tm* localAsTm = localtime(&secsSinceEpoch);
-        //Rcpp::Rcout << "Adj is " << localAsTm->tm_isdst << std::endl;
-
-        // Define BOOST_DATE_TIME_POSIX_TIME_STD_CONFIG to use nanoseconds
-        // (and then use diff.total_nanoseconds()/1.0e9;  instead)
-        //
-        // note dst correction here -- needed as UTC offset is correct but does not
-        // contain the additional DST adjustment
-        return tdiff.total_microseconds()/1.0e6 - localAsTm->tm_isdst*60*60;
-    }
+    return (pt == ptbase) ? NAN : ptToDouble(pt);
 }
 
 template <int RTYPE>
@@ -221,3 +234,25 @@ void addFormats(Rcpp::CharacterVector fmt) {
         timeformats.addFormat(fmt[i]);
     }
 }
+
+
+// [[Rcpp::export]]
+Rcpp::NumericVector testFormat(const std::string fmt, const std::string s, const std::string tz = "") {
+    
+    bt::ptime pt, ptbase;
+
+    std::istringstream is(s);
+    std::locale loc = std::locale(std::locale::classic(), new bt::time_input_facet(fmt));
+    
+    is.imbue(loc);
+    is >> pt;
+
+    double timeval = (pt == ptbase) ? NAN : ptToDouble(pt);
+
+    Rcpp::NumericVector pv(1);
+    pv(0) = timeval;
+    pv.attr("class") = Rcpp::CharacterVector::create("POSIXct", "POSIXt");
+    pv.attr("tzone") = tz;
+    
+    return pv;
+}
